﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Windows.Interactivity;
using Infragistics.Controls.Editors;
using Infragistics.Controls.Editors.Primitives;

namespace WpfApplication1
{
    public class XamComboEditorBehavior : Behavior<XamComboEditor>
    {
        protected override void OnAttached()
        {
            base.OnAttached();

            AssociatedObject.Loaded += AssociatedObject_Loaded;
        }

        void AssociatedObject_Loaded(object sender, System.Windows.RoutedEventArgs e)
        {
            AssociatedObject.Loaded -= AssociatedObject_Loaded;

            SpecializedTextBox tb =
                (SpecializedTextBox)Infragistics.Windows.Utilities.GetDescendantFromType(AssociatedObject, typeof(SpecializedTextBox), false);

            if (tb != null)
                tb.TextChanged += tb_TextChanged;
        }
        protected override void OnDetaching()
        {
            SpecializedTextBox tb =
                (SpecializedTextBox)Infragistics.Windows.Utilities.GetDescendantFromType(AssociatedObject, typeof(SpecializedTextBox), false);

            if (tb != null)
                tb.TextChanged -= tb_TextChanged;

            base.OnDetaching();
        }

        void tb_TextChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            SpecializedTextBox tb = sender as SpecializedTextBox;

            if (string.IsNullOrEmpty(tb.Text))
                return;

            Type itemType = null;
            // does the value in the text box associate with any in the combo editor?
            foreach (object item in AssociatedObject.ItemsSource)
            {
                itemType = item.GetType();

                PropertyInfo pInfo = item.GetType().GetProperty(AssociatedObject.DisplayMemberPath);
                if (pInfo != null)
                    if (tb.Text == (string)pInfo.GetValue(item, null))
                        AssociatedObject.SelectedItem = item;
            }

            // still haven't found an item in the list
            if (AssociatedObject.SelectedItem == null && itemType != null)
            {
                var newItem = Activator.CreateInstance(itemType);
                PropertyInfo pInfo = newItem.GetType().GetProperty(AssociatedObject.DisplayMemberPath);
                if (pInfo != null)
                    pInfo.SetValue(newItem, tb.Text, null);

                AssociatedObject.SelectedItem = newItem;
            }
        }
    }
}
